"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const console_out_js_1 = require("plop/src/console-out.js");
const argv_1 = require("../argv");
const errors_1 = require("../errors");
const store_1 = require("../store");
const types_1 = require("../types");
const actions_1 = require("./actions");
const generator_1 = require("./actions/generator");
const bypass_1 = tslib_1.__importDefault(require("./bypass"));
const generators_1 = require("./generators");
const messages_1 = require("./messages");
const setPrompts_1 = require("./prompts/setPrompts");
const tasks_1 = require("./tasks");
exports.default = (plop) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    /**
     * Set defaults
     */
    (0, actions_1.setActionTypes)(plop);
    (0, setPrompts_1.setPrompts)(plop);
    /**
     * Capture runtime args
     */
    const { bypassArr, generatorName } = (0, bypass_1.default)(plop);
    /**
     * Init previous answers
     */
    const prevAns = {};
    /**
     * authenticate user and retrieve entitlements
     */
    const credentials = yield (0, tasks_1.authenticateUser)(inquirer_1.default, prevAns);
    yield (0, tasks_1.setCredentials)(credentials);
    const entitlements = yield (0, tasks_1.downloadEntitlements)();
    /**
     * Add generators based on user entitlements
     */
    if (!store_1.configStore.values.external && entitlements.has(types_1.Entitlement.appCreate)) {
        const { name, generator } = (0, generators_1.getDSLGenerator)(plop, bypassArr);
        plop.setGenerator(name, generator);
    }
    if (entitlements.has(types_1.Entitlement.workspaceCreate)) {
        const { name, generator } = (0, generators_1.getWorkspaceGenerator)(plop, bypassArr);
        plop.setGenerator(name, generator);
    }
    if (entitlements.has(types_1.Entitlement.workspaceConfigure)) {
        plop.setGenerator('configure workspace', generators_1.configureWorkspaceGenerator);
    }
    if (entitlements.has(types_1.Entitlement.appCreate)) {
        const { name, generator } = (0, generators_1.getAppGenerator)(plop, bypassArr);
        plop.setGenerator(name, generator);
    }
    if (entitlements.has(types_1.Entitlement.appConfigure)) {
        plop.setGenerator('configure application', generators_1.configureAppGenerator);
    }
    if (entitlements.has(types_1.Entitlement.foundationSync)) {
        plop.setGenerator('foundation', generators_1.foundationGenerator);
    }
    if (entitlements.has(types_1.Entitlement.foundationComponentCreate)) {
        plop.setGenerator('component', generators_1.componentGenerator);
    }
    if (entitlements.has(types_1.Entitlement.designSystemCreate)) {
        plop.setGenerator('design system', generators_1.designSystemGenerator);
    }
    /**
     * Add generators that are available to all
     */
    const { name: switchName, generator: switchGenerator } = (0, generators_1.getSwitchDesignSystemGenerator)(plop, bypassArr);
    plop.setGenerator(switchName, switchGenerator);
    plop.setGenerator('analyze component usage', generators_1.analyzeComponentUsageGenerator);
    const generators = plop.getGeneratorList();
    if (!generators.length) {
        throw new Error(`No runnable commands available, please check your entitlements.`);
    }
    /**
     * Run default / only generator. If they haven't specified a target generator and there's only one.
     */
    if (!generatorName && generators.length === 1) {
        yield generator_1.GeneratorActions.run(plop.getGenerator(generators[0].name), prevAns, plop, bypassArr);
        return;
    }
    /**
     * Prompt to pick a generator: If they haven't specified a target generator and there's multiple.
     */
    if (!generatorName && generators.length > 1) {
        (0, console_out_js_1.chooseOptionFromList)(plop.getGeneratorList(), (0, messages_1.firstStep)())
            .then((name) => generator_1.GeneratorActions.run(plop.getGenerator(name), prevAns, plop, bypassArr))
            .catch((error) => {
            (0, errors_1.errorHandler)(error);
        });
        return;
    }
    /**
     * Run target generator: If they specified a target generator and it exists.
     */
    const generator = plop.getGenerator(generatorName);
    if (generator) {
        yield generator_1.GeneratorActions.run(generator, prevAns, plop, bypassArr);
        return;
    }
    /**
     * Not sure what to do with this input
     */
    const fuzzyGeneratorName = `${generatorName}  ${argv_1.args.join(' ')}`.trim();
    throw new Error(`Not sure what to run with input args: ${fuzzyGeneratorName}`);
});
